/*
 * 29-04-2016 LOCO Wi-Fi - 2
 * 31-03-2017 LOCO Wi-Fi - 3
 *  server_ip is the IP address of the ESP8266 module, will be 
 *  printed to Serial when the module is connected.
 */
#include <ESP8266WiFi.h>
#include <SPI.h>
#include "MFRC522.h"
/* wiring the MFRC522 to ESP8266 (ESP-12)
*  RST     = GPIO2
*  SDA(SS) = GPIO15
*  MOSI    = GPIO13
*  MISO    = GPIO12
*  SCK     = GPIO14
*  GND     = GND
*  3.3V    = 3.3V
*/
#define RST_PIN  2 // RST-PIN for RC522 - RFID - SPI - GPIO2
#define SS_PIN  15 // SDA-PIN for RC522 - RFID - SPI - GPIO15

#define UID_LEN 7

#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

// called this way, it uses the default address 0x40
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();
// you can also call it with a different address you want
//Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver(0x41);

// Depending on your servo make, the pulse width min and max may vary, you 
// want these to be as small/large as possible without hitting the hard stop
// for max range. You'll have to tweak them as necessary to match the servos you
// have!
#define SERVOMIN  150 // this is the 'minimum' pulse length count (out of 4096)
#define SERVOMAX  500 // this is the 'maximum' pulse length count (out of 4096)

// WLAN Default config:
IPAddress Ip(192, 168, 0, 111); //default IP
IPAddress Gw(192, 168, 0, 1); //default Gateway
IPAddress Sb(255, 255, 255, 0); //default Subnet
// Your network name and password.
const char* ssid = "Z21_6666";
const char* password = "54448049";

// Create an instance of the server
// specify the port to listen on as an argument 4711
WiFiServer server(4711);

boolean alreadyConnected = false; // whether or not the client was connected previously

MFRC522 mfrc522(SS_PIN, RST_PIN); // Create MFRC522 instance
MFRC522::MIFARE_Key key;

bool bRFIDactive;
unsigned char i = 0;
unsigned char j = 0;
uint32_t uiDelayTime = 500;
uint32_t RFIDQuietAfterTag = 2000;

uint32_t RFIDCycle;
bool bRFIDquiet;

bool SensorOutput_Inactive = true;

byte oldUid[7];

//--- LOCO ---
byte speed = 0;
bool bool_dir = false;
bool bool_f0 = false;
bool bool_f1 = false;
bool bool_f2 = false;
bool bool_f3 = false;

void setup() {
  Serial.begin(115200);

  SPI.begin();  // Init SPI bus
  mfrc522.PCD_Init(); // Init MFRC522

  // Prepare key - all keys are set to FFFFFFFFFFFFh at chip delivery from the factory.
  for (byte i = 0; i < 6; i++) {
    key.keyByte[i] = 0xFF;
  }

  Wire.begin(4,5);  // (SDA on 4, SCL on 5) - (with an ESP, GPIO4 & GPIO5).

  pwm.begin();
  
  pwm.setPWMFreq(60);  // Analog servos run at ~60 Hz updates

  yield();

  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.config(Ip,Gw,Sb);
  WiFi.begin(ssid, password);
  
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");
  
  // Start the server
  server.begin();
  Serial.println("Server started");

  // Print the IP address
  Serial.println(WiFi.localIP());

  Serial.println("start end");
}

void loop() {
  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) {
    return;
  }
 
  Serial.println("new client");  // Wait until the client sends some data

  while(!client.available()){

    if ( mfrc522.PICC_IsNewCardPresent() && mfrc522.PICC_ReadCardSerial()) {
     //Serial.println("ReadCard");
      if (!compareUid( mfrc522.uid.uidByte, oldUid, mfrc522.uid.size)) { // NEW  (NOT same UID)
        if (!((oldUid[1] == 0) && (oldUid[2] == 0))) {
          //Serial.println("!compareUid");
          SensorOutput_Inactive = true;
//----------------
          String url = "id=\"";  
          url += oldUid[0];
          url += ".";
          url += oldUid[1];
          url += ".";
          url += oldUid[2];
          url += ".";
          url += oldUid[3]; 
          //Serial.println(String("<fb ") + url + "\" state=\"false\" fbtype=\"5\"/>");
          client.println(String("<fb ") + url + "\" state=\"false\" fbtype=\"5\"/>");  
//------------
          oldUid[0] = 0;
          oldUid[1] = 0;
          oldUid[2] = 0;
          oldUid[3] = 0;
        }
        
        //SendUID - (mfrc522.uid.uidByte, 1, 1)
        SensorOutput_Inactive = false;
//----------------
        String url = "id=\"";  
        url += mfrc522.uid.uidByte[0];
        url += ".";
        url += mfrc522.uid.uidByte[1];
        url += ".";
        url += mfrc522.uid.uidByte[2];
        url += ".";
        url += mfrc522.uid.uidByte[3]; 
        //Serial.println(String("<fb ") + url + "\" state=\"true\" fbtype=\"5\"/>");
        client.println(String("<fb ") + url + "\" state=\"true\" fbtype=\"5\"/>");
//------------
        oldUid[0] = 0;
        oldUid[1] = 0;
        oldUid[2] = 0;
        oldUid[3] = 0;
             
        RFIDCycle = millis() + RFIDQuietAfterTag; // switched on, so do'nt do any more reading for 2 secs..
        copyUid(oldUid, mfrc522.uid.uidByte,  mfrc522.uid.size); //save in oldUID
        //     uiStartTime = millis()+RFIDQuietAfterTag;
        //     bRFIDactive=true;
        bRFIDquiet = false;
        //Serial.println("Found new card..");
      }//  end NOT same UID same
      else { //is the same card still...
           // Serial.println("Same card..");
           RFIDCycle = millis() + RFIDQuietAfterTag; //update time card was seen
      }
     
   } else { //is the same card still...
 
      if ((!bRFIDquiet) && (millis() >= RFIDCycle + uiDelayTime)) {
        // Serial.println("Nothing read for a while."); // wait delay before declaring the card gone
        // SendUID - (oldUid, 0, 3)
//----------------
        String url = "id=\"";  
        url += oldUid[0];
        url += ".";
        url += oldUid[1];
        url += ".";
        url += oldUid[2];
        url += ".";
        url += oldUid[3]; 
        //Serial.println(String("<fb ") + url + "\" state=\"false\" fbtype=\"5\"/>");
        client.println(String("<fb ") + url + "\" state=\"false\" fbtype=\"5\"/>");
//------------
        oldUid[0] = 0;
        oldUid[1] = 0;
        oldUid[2] = 0;
        oldUid[3] = 0;
        
        bRFIDquiet = true;
      }
    }
    // delay(1);
  } // --- END while

  //--- chtenie stroki
 String str = client.readStringUntil('/');

 client.stop();
 Serial.println("client disonnected");

 //--- obrabotka
 char *clientline = &str[0]; //string -> char

 // Print it out for debugging
 Serial.println("------------------------------------");
 Serial.print("clientline:");
 Serial.println(clientline);

 // parse commands
 if (strstr(clientline, "<fn ") != 0) {
   if (strstr(clientline, "V200") != 0) {
     Serial.println("Fn V200");
     char *get_arg;
     get_arg = strstr(clientline, "V200");
     //f0
     bool_f0 = ((strstr(get_arg, "f0=")+4)[0] == 't');
     Serial.print("F-0=");
     Serial.println(bool_f0);
     //f1
     bool_f1 = ((strstr(get_arg, "f1=")+4)[0] == 't');
     Serial.print("F-1=");               
     Serial.println(bool_f1);
     //f2
     bool_f2 = ((strstr(get_arg, "f2=")+4)[0] == 't');
     Serial.print("F-2=");               
     Serial.println(bool_f2);
     //f3
     bool_f3 = ((strstr(get_arg, "f3=")+4)[0] == 't');
     Serial.print("F-3=");
     Serial.println(bool_f3);
     // ispolnenie
     set_lights();
   } 
 } 
 else if (strstr(clientline, "<lc ") != 0) {
        char *locname;
        locname = clientline + 4;
        Serial.print("Locname:");
        Serial.println(locname);
          // parse commands       
          if (strstr(locname, "V200") != 0) {
            Serial.println("play track V200");
            char *get_arg;
            get_arg = strstr(locname, "V=");
            speed = atol(get_arg + 3);
            Serial.print("Speed=");
            Serial.println(speed, DEC);
            //DIR
            bool_dir = ((strstr(get_arg, "dir=")+5)[0] == 't');
            Serial.print("DIR=");
            Serial.println(bool_dir);
            //f0
            bool_f0 = ((strstr(get_arg, "f0=")+4)[0] == 't');
            Serial.print("F-0=");
            Serial.println(bool_f0);
            //f1
            bool_f1 = ((strstr(get_arg, "f1=")+4)[0] == 't');
            Serial.print("F-1=");               
            Serial.println(bool_f1);
            //f2
            bool_f2 = ((strstr(get_arg, "f2=")+4)[0] == 't');
            Serial.print("F-2=");               
            Serial.println(bool_f2);
            //f3
            bool_f3 = ((strstr(get_arg, "f3=")+4)[0] == 't');
            Serial.print("F-3=");
            Serial.println(bool_f3);
            // ispolnenie
            set_lights();
          } 
 }
} //END loop

/*
   Function used to compare two RFID UIDs
*/
bool compareUid(byte *buffer1, byte *buffer2, byte bufferSize) {
  for (byte i = 0; i < bufferSize; i++) {
    if (buffer1[i] != buffer2[i]) {
      return (false);
    }
  }
  return (true);
}

void copyUid (byte *buffOut, byte *buffIn, byte bufferSize) {
  for (byte i = 0; i < bufferSize; i++) {
    buffOut[i] = buffIn[i];
  }
  if (bufferSize < UID_LEN) {
    for (byte i = bufferSize; i < UID_LEN; i++) {
      buffOut[i] = 0;
    }
  }
}

void set_lights() {
  if (bool_f0 == true) { pwm.setPWM( 0, 0, SERVOMAX); } else { pwm.setPWM( 0, 0, SERVOMIN); }
  if (bool_f1 == true) { pwm.setPWM( 1, 0, SERVOMAX); } else { pwm.setPWM( 1, 0, SERVOMIN); }
  if (bool_f2 == true) { pwm.setPWM( 2, 0, SERVOMAX); } else { pwm.setPWM( 2, 0, SERVOMIN); }
  if (bool_f3 == true) { pwm.setPWM( 3, 0, SERVOMAX); } else { pwm.setPWM( 3, 0, SERVOMIN); }
}

